"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const concept_helper_1 = require("./concept-helper");
const name_helper_1 = require("../name-helper");
const actor_name_collection_1 = require("./actor-name-collection");
class ActorHelper {
    static build(locale, actorNames, wikiEntity) {
        const lang = locale.lang.trim().toLowerCase();
        const country = locale.country.trim().toLowerCase();
        actorNames = ActorHelper.buildNames(lang, actorNames, wikiEntity && wikiEntity.names);
        const names = actorNames.list();
        if (!names.length) {
            throw new Error(`Invalid ConceptActor: no names!`);
        }
        const name = wikiEntity && wikiEntity.name || names[0].name;
        const actor = {
            lang,
            country,
            name,
            wikiEntity,
            names,
        };
        const commonName = ActorHelper.findCommonName(name, names);
        if (commonName) {
            actor.commonName = commonName;
        }
        const abbr = ActorHelper.findAbbr(name, names);
        if (abbr) {
            actor.abbr = abbr;
        }
        return actor;
    }
    static findAbbr(name, names) {
        const abbr = names.find(item => item.isAbbr && item.popularity > 1 && item.type === 'WIKI' && name.length > item.name.length);
        if (abbr) {
            return abbr.name;
        }
        return null;
    }
    static findCommonName(name, names) {
        const nameCountWords = name_helper_1.NameHelper.countWords(name);
        if (nameCountWords < 3) {
            return null;
        }
        const popularName = names.find(item => !item.isAbbr && item.popularity > 1 && ActorHelper.isValidCommonName(item.name));
        if (!popularName || popularName.popularity < 10) {
            return null;
        }
        const popularNameCountWords = name_helper_1.NameHelper.countWords(names[0].name);
        if (popularNameCountWords < 2) {
            return null;
        }
        if (nameCountWords <= popularNameCountWords) {
            return null;
        }
        return popularName.name;
    }
    static buildNames(lang, nameCollection, entityNames) {
        if (entityNames) {
            const collection = new actor_name_collection_1.ActorNameCollection(lang);
            for (const name of nameCollection.list()) {
                collection.add(name);
            }
            for (const name of entityNames) {
                collection.add({ name, popularity: 0, type: 'WIKI' });
            }
            nameCollection = collection;
        }
        return nameCollection;
    }
    static validate(entity) {
        if (!entity) {
            throw new Error(`Invalid ConceptActor: null or undefined`);
        }
        if (!entity.lang) {
            throw new Error(`Invalid ConceptActor: invalid lang`);
        }
        if (!entity.country) {
            throw new Error(`Invalid ConceptActor: invalid country`);
        }
        if (!concept_helper_1.ConceptHelper.isValidName(entity.name, entity.lang)) {
            throw new Error(`Invalid ConceptActor: invalid name: ${entity.name}`);
        }
        if (!entity.names || !entity.names.length) {
            throw new Error(`Invalid ConceptActor: no names`);
        }
        const invalidName = entity.names.find(item => !concept_helper_1.ConceptHelper.isValidName(item.name, entity.lang));
        if (invalidName) {
            throw new Error(`Invalid ConceptActor: names contains invalid names: ${invalidName}`);
        }
    }
    static isValidCommonName(name) {
        return !/[,(‒–—―«»"“”‘’;:]/.test(name);
    }
}
exports.ActorHelper = ActorHelper;
