"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const wiki_entity_1 = require("wiki-entity");
const wiki_entity_2 = require("./wiki-entity");
const partial_name_1 = require("partial-name");
const concept_helper_1 = require("./concept-helper");
const name_helper_1 = require("../name-helper");
const utils_1 = require("../utils");
class WikiEntityHelper {
    static getPartialName(name, lang, entityName) {
        if (!name || name_helper_1.NameHelper.countWords(name) < 2) {
            return null;
        }
        const exResult = /\(([^)]+)\)$/.exec(name);
        let partial = null;
        if (exResult) {
            partial = name.substr(0, exResult.index).trim();
            if (name_helper_1.NameHelper.countWords(partial) < 2) {
                partial = null;
            }
        }
        if (!partial) {
            partial = partial_name_1.partialName(name, { lang });
            if (partial && name_helper_1.NameHelper.countWords(partial) < 2) {
                partial = null;
            }
        }
        if (partial) {
            const partialFirstWord = name_helper_1.NameHelper.atonic(partial.split(/\s+/)[0].toLowerCase());
            const entityNameFirstWord = name_helper_1.NameHelper.atonic(entityName.split(/\s+/)[0].toLowerCase());
            if (partialFirstWord !== entityNameFirstWord) {
                return null;
            }
        }
        return partial;
    }
    static getName(entity) {
        if (!entity.wikiPageTitle) {
            throw new Error(`wikiPageTitle is required!`);
        }
        return entity.wikiPageTitle;
    }
    static isValidName(name, lang) {
        return concept_helper_1.ConceptHelper.isValidName(name, lang);
    }
    static nameHash(name, lang) {
        lang = lang.trim().toLowerCase();
        name = name.trim();
        name = name_helper_1.NameHelper.normalizeName(name, lang);
        name = name_helper_1.NameHelper.atonic(name);
        return utils_1.md5([lang, name].join('_'));
    }
    static namesHashes(names, lang) {
        names = names.filter(name => WikiEntityHelper.isValidName(name, lang));
        const hashes = utils_1.uniq(names.map(name => WikiEntityHelper.nameHash(name, lang)));
        return hashes;
    }
    static rootName(name, lang) {
        return concept_helper_1.ConceptHelper.rootName(name, lang);
    }
    static rootNameHash(name, lang) {
        return WikiEntityHelper.nameHash(WikiEntityHelper.rootName(name, lang), lang);
    }
    static convertSimpleEntityType(type) {
        switch (type) {
            case wiki_entity_1.SimpleEntityType.EVENT: return wiki_entity_2.WikiEntityType.EVENT;
            case wiki_entity_1.SimpleEntityType.ORG: return wiki_entity_2.WikiEntityType.ORG;
            case wiki_entity_1.SimpleEntityType.PERSON: return wiki_entity_2.WikiEntityType.PERSON;
            case wiki_entity_1.SimpleEntityType.PLACE: return wiki_entity_2.WikiEntityType.PLACE;
            case wiki_entity_1.SimpleEntityType.PRODUCT: return wiki_entity_2.WikiEntityType.PRODUCT;
            case wiki_entity_1.SimpleEntityType.WORK: return wiki_entity_2.WikiEntityType.WORK;
        }
    }
    static splitName(name) {
        const firstIndex = name.indexOf('(');
        if (firstIndex < 3) {
            return null;
        }
        const lastIndex = name.indexOf(')');
        if (lastIndex !== name.length - 1) {
            return null;
        }
        return {
            simple: name.substr(0, firstIndex).trim(),
            special: name.substring(firstIndex + 1, lastIndex)
        };
    }
    static getSimpleName(name) {
        const splitName = WikiEntityHelper.splitName(name);
        if (splitName) {
            return splitName.simple;
        }
    }
    static isDisambiguation(entity) {
        return entity && entity.data && entity.data.P31 && entity.data.P31.indexOf('Q4167410') > -1;
    }
    static getPopularity(rank) {
        if (!rank || rank < 0) {
            return EntityPopularity.UNKNOWN;
        }
        const r = rank / 10;
        if (r < 2) {
            return EntityPopularity.UNKNOWN;
        }
        if (r < 4) {
            return EntityPopularity.LOW;
        }
        if (r < 6) {
            return EntityPopularity.NORMAL;
        }
        if (r < 9) {
            return EntityPopularity.HIGH;
        }
        return EntityPopularity.POPULAR;
    }
    static isNotActual(entity) {
        if (!entity.data) {
            return undefined;
        }
        const notActualProps = ['P457', 'P20', 'P576'];
        const props = Object.keys(entity.data);
        for (let prop of notActualProps) {
            if (~props.indexOf(prop)) {
                return true;
            }
        }
        return undefined;
    }
}
exports.WikiEntityHelper = WikiEntityHelper;
var EntityPopularity;
(function (EntityPopularity) {
    EntityPopularity[EntityPopularity["UNKNOWN"] = 1] = "UNKNOWN";
    EntityPopularity[EntityPopularity["LOW"] = 2] = "LOW";
    EntityPopularity[EntityPopularity["NORMAL"] = 3] = "NORMAL";
    EntityPopularity[EntityPopularity["HIGH"] = 4] = "HIGH";
    EntityPopularity[EntityPopularity["POPULAR"] = 5] = "POPULAR";
})(EntityPopularity = exports.EntityPopularity || (exports.EntityPopularity = {}));
