"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
class MemoryConceptRepository {
    constructor() {
        this.db = new Map();
    }
    getMostPopular(containerId, limit, skip, options) {
        options = Object.assign({}, options);
        const list = [];
        for (let item of this.db.values()) {
            if (item.containerId !== containerId) {
                continue;
            }
            if (options.minCountWords && item.countWords < options.minCountWords) {
                continue;
            }
            if (options.maxCountWords && item.countWords > options.maxCountWords) {
                continue;
            }
            list.push(item);
        }
        return Promise.resolve(list.slice(skip, skip + limit));
    }
    getByRootNameIds(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            let list = [];
            for (let id of ids) {
                const concepts = yield this.getByRootNameId(id);
                list = list.concat(concepts);
            }
            return utils_1.uniqByProp(list, 'id');
        });
    }
    count(locale) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.country !== locale.country || item.lang !== locale.lang) {
                continue;
            }
            count++;
        }
        return Promise.resolve(count);
    }
    getConceptsWithAbbr(containerId) {
        const list = [];
        for (let item of this.db.values()) {
            if (item.containerId === containerId && !item.isAbbr && item.abbr) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    deleteByRootNameIds(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const list = yield this.getByRootNameIds(ids);
            let count = list.length;
            for (let item of list) {
                this.db.delete(item.id);
            }
            return Promise.resolve(count);
        });
    }
    list(locale, limit, skip) {
        skip = skip || 0;
        const list = [];
        for (let item of this.db.values()) {
            if (item.country !== locale.country || item.lang !== locale.lang) {
                continue;
            }
            list.push(item);
        }
        return Promise.resolve(list.slice(skip, skip + limit));
    }
    getById(id) {
        return Promise.resolve(this.db.get(id) || null);
    }
    getByIds(ids) {
        const list = [];
        for (let id of ids) {
            const item = this.db.get(id);
            if (item) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    exists(id) {
        return Promise.resolve(!!this.db.get(id));
    }
    delete(id) {
        return Promise.resolve(this.db.delete(id));
    }
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!!this.db.get(data.id)) {
                return Promise.reject(new Error(`Item already exists!`));
            }
            this.db.set(data.id, Object.assign({ popularity: 1, createdAt: Date.now() }, data));
            const entity = yield this.getById(data.id);
            if (!entity) {
                throw new Error(`Entity not found!`);
            }
            return entity;
        });
    }
    update(data) {
        const item = this.db.get(data.id);
        if (!item) {
            return Promise.reject(new Error(`Item not found! id=${data.id}`));
        }
        if (data.set) {
            for (let prop in data.set) {
                if ([null, undefined].indexOf(data.set[prop]) < 0) {
                    item[prop] = data.set[prop];
                }
            }
        }
        if (data.delete) {
            for (let prop of data.delete) {
                delete item[prop];
            }
        }
        return Promise.resolve(item);
    }
    getByRootNameId(id) {
        const list = [];
        for (let item of this.db.values()) {
            if (~item.rootNameIds.indexOf(id)) {
                list.push(item);
            }
        }
        return Promise.resolve(list);
    }
    deleteUnpopular(containerId, popularity) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.containerId !== containerId) {
                continue;
            }
            if (item.popularity <= popularity) {
                this.db.delete(item.id) && count++;
            }
        }
        return Promise.resolve(count);
    }
    deleteUnpopularAbbreviations(containerId, popularity) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.containerId !== containerId) {
                continue;
            }
            if (item.isAbbr && item.popularity <= popularity) {
                this.db.delete(item.id) && count++;
            }
        }
        return Promise.resolve(count);
    }
    deleteUnpopularOneWorlds(containerId, popularity) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.containerId !== containerId) {
                continue;
            }
            if (item.countWords === 1 && item.popularity <= popularity) {
                this.db.delete(item.id) && count++;
            }
        }
        return Promise.resolve(count);
    }
    deleteAll(containerId) {
        let count = 0;
        for (let item of this.db.values()) {
            if (item.containerId !== containerId) {
                continue;
            }
            this.db.delete(item.id) && count++;
        }
        return Promise.resolve(count);
    }
    deleteIds(ids) {
        let count = 0;
        for (let id of ids) {
            this.db.delete(id) && count++;
        }
        return Promise.resolve(count);
    }
    incrementPopularity(id) {
        const item = this.db.get(id);
        if (!item) {
            return Promise.resolve(0);
        }
        item.popularity++;
        return Promise.resolve(item.popularity);
    }
    createOrUpdate(concept) {
        return __awaiter(this, void 0, void 0, function* () {
            concept = Object.assign({}, concept);
            const id = concept.id;
            let item = this.db.get(id);
            if (!item) {
                item = yield this.create(concept);
            }
            else {
                item.popularity++;
                item.rootNameIds = utils_1.uniq(item.rootNameIds.concat(concept.rootNameIds));
            }
            return Promise.resolve(item);
        });
    }
    all() {
        const array = [];
        for (let item of this.db.values()) {
            array.push(item);
        }
        return Promise.resolve(array);
    }
}
exports.MemoryConceptRepository = MemoryConceptRepository;
