"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('textactor-explorer');
const wiki_entity_1 = require("wiki-entity");
const wiki_entity_helper_1 = require("../../entities/wiki-entity-helper");
const text_country_1 = require("text-country");
const utils_1 = require("../../utils");
const name_helper_1 = require("../../name-helper");
class WikiEntityBuilder {
    constructor(locale, knownNamesService) {
        this.locale = locale;
        this.knownNamesService = knownNamesService;
    }
    build({ wikiEntity }) {
        const lang = this.locale.lang.trim().toLowerCase();
        const country = this.locale.country.trim().toLowerCase();
        const simpleEntity = wiki_entity_1.convertToSimpleEntity(wikiEntity, lang);
        const name = wiki_entity_helper_1.WikiEntityHelper.getName(simpleEntity);
        if (!name) {
            throw new Error(`Entity has no name! ${simpleEntity.wikiDataId}`);
        }
        const entity = {
            id: `${lang.trim().toUpperCase()}${simpleEntity.wikiDataId}`,
            name: name,
            names: [],
            namesHashes: [],
            partialNames: [],
            partialNamesHashes: [],
            lang: lang,
            description: simpleEntity.description,
            aliases: utils_1.uniq(wikiEntity.aliases || []),
            about: simpleEntity.about,
            wikiDataId: simpleEntity.wikiDataId,
            wikiPageId: simpleEntity.wikiPageId,
            wikiPageTitle: simpleEntity.wikiPageTitle,
            types: simpleEntity.types,
            countryCodes: simpleEntity.countryCodes || [],
            data: simpleEntity.data,
            categories: simpleEntity.categories,
            rank: 1,
            links: {},
        };
        if (!entity.countryCodes.length) {
            const code = getCountryByTitle(utils_1.filterStrings([entity.wikiPageTitle, entity.name]), lang);
            if (code) {
                entity.countryCodes = [code];
            }
        }
        const knownName = this.knownNamesService.getKnownName(name, lang, country);
        if (knownName && knownName.countryCodes) {
            debug(`Adding countryCodes for a known name: ${name}`);
            entity.countryCodes = utils_1.uniq((entity.countryCodes || []).concat(knownName.countryCodes));
        }
        if (simpleEntity.type) {
            entity.type = wiki_entity_helper_1.WikiEntityHelper.convertSimpleEntityType(simpleEntity.type);
        }
        if (simpleEntity.abbr) {
            entity.abbr = simpleEntity.abbr;
        }
        entity.rank += entity.aliases.length;
        if (wikiEntity.sitelinks) {
            entity.rank += Object.keys(wikiEntity.sitelinks).length * 5;
            entity.links = wikiEntity.sitelinks;
        }
        if (entity.data) {
            entity.rank += Object.keys(entity.data).length;
        }
        let splittedName = wiki_entity_helper_1.WikiEntityHelper.splitName(entity.name);
        if (splittedName) {
            entity.specialName = splittedName.special;
            entity.simpleName = splittedName.simple;
        }
        else if (entity.wikiPageTitle) {
            splittedName = wiki_entity_helper_1.WikiEntityHelper.splitName(entity.wikiPageTitle);
            if (splittedName) {
                entity.specialName = splittedName.special;
                entity.simpleName = splittedName.simple;
            }
        }
        entity.names = [entity.name, entity.wikiPageTitle]
            .filter(name => wiki_entity_helper_1.WikiEntityHelper.isValidName(name, lang));
        if (wikiEntity.redirects && wikiEntity.redirects.length) {
            entity.names = entity.names.concat(wikiEntity.redirects);
            if (!entity.abbr) {
                const abbr = name_helper_1.NameHelper.findAbbr(wikiEntity.redirects);
                if (abbr) {
                    entity.abbr = abbr;
                }
            }
        }
        entity.names = entity.names.map(name => name_helper_1.NameHelper.standardText(name, lang));
        entity.names = entity.names.filter(name => wiki_entity_helper_1.WikiEntityHelper.isValidName(name, lang));
        entity.names = utils_1.uniq(entity.names);
        entity.namesHashes = wiki_entity_helper_1.WikiEntityHelper.namesHashes(entity.names, lang);
        let partialNames = entity.names.map(name => wiki_entity_helper_1.WikiEntityHelper.getPartialName(name, lang, entity.name))
            .filter(name => name && name_helper_1.NameHelper.countWords(name) > 1 && entity.names.indexOf(name) < 0);
        partialNames = utils_1.uniq(partialNames);
        if (partialNames.length) {
            entity.partialNames = partialNames;
            entity.partialNamesHashes = wiki_entity_helper_1.WikiEntityHelper.namesHashes(entity.partialNames, lang);
            entity.partialNamesHashes = utils_1.uniq(entity.partialNamesHashes);
        }
        for (let name of Object.keys(entity)) {
            if (~[null, undefined, ''].indexOf(entity[name])) {
                delete entity[name];
            }
        }
        return entity;
    }
}
exports.WikiEntityBuilder = WikiEntityBuilder;
function getCountryByTitle(titles, lang) {
    const title = titles.filter(item => !!item).join('; ');
    const result = text_country_1.default(title, lang);
    if (result && result.length) {
        return result[0].country.trim().toLowerCase();
    }
    return null;
}
